@@ -20,6 +20,7 @@
  * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
  */
 
+#include <string.h>
 #include "e-book-backend-couchdb.h"
 #include <libedata-book/e-data-book.h>
 #include <libedata-book/e-data-book-view.h>
@@ -362,7 +363,7 @@ vcard_from_couch_document (CouchDBDocument *document)
 
 	/* birth date */
 	str = (char *) couchdb_document_contact_get_birth_date (document);
-	if (str) {
+	if (str && strlen (str) > 0) {
 		EContactDate *dt;
 
 		dt = e_contact_date_from_string (str);
@@ -374,7 +375,7 @@ vcard_from_couch_document (CouchDBDocument *document)
 
 	/* wedding date */
 	str = (char *) couchdb_document_contact_get_wedding_date (document);
-	if (str) {
+	if (str && strlen (str)) {
 		EContactDate *dt;
 
 		dt = e_contact_date_from_string (str);
@@ -908,7 +909,7 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 			uri = g_strdup_printf ("http://127.0.0.1:%d", port);
 
 			/* Get OAuth tokens from GnomeKeyring */
-			attrs = gnome_keyring_attribute_list_new();
+			attrs = gnome_keyring_attribute_list_new ();
 			gnome_keyring_attribute_list_append_string (attrs, "desktopcouch", "oauth");
 
 			result = gnome_keyring_find_items_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
@@ -1026,15 +1027,40 @@ e_book_backend_couchdb_get_static_capabilities (EBookBackend *backend)
 	return g_strdup ("local,do-initial-query,bulk-removes,contact-lists");
 }
 
+static EContact *
+put_document (EBookBackendCouchDB *couchdb_backend, CouchDBDocument *document)
+{
+	GError *error = NULL;
+
+	if (couchdb_document_put (document, couchdb_backend->dbname, &error)) {
+		EContact *new_contact;
+		char *vcard_str;
+
+		/* couchdb_document_put sets the ID for new documents, so need to send that back */
+		vcard_str = vcard_from_couch_document (document);
+		new_contact = e_contact_new_from_vcard (vcard_str);
+
+		g_free (vcard_str);
+
+		return new_contact;
+	} else {
+		if (error != NULL) {
+			g_warning ("Could not PUT document: %s\n", error->message);
+			g_error_free (error);
+		}
+	}
+
+	return NULL;
+}
+
 static void
 e_book_backend_couchdb_create_contact (EBookBackend *backend,
 				       EDataBook *book,
 				       guint32 opid,
 				       const char *vcard)
 {
-	EContact *contact;
+	EContact *contact, *new_contact;
 	CouchDBDocument *document;
-	GError *error = NULL;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	contact = e_contact_new_from_vcard (vcard);
@@ -1051,17 +1077,8 @@ e_book_backend_couchdb_create_contact (EBookBackend *backend,
 	}
 
 	/* save the contact into the DB */
-	if (couchdb_document_put (document, couchdb_backend->dbname, &error)) {
-		EContact *new_contact;
-		char *vcard_str;
-
-		/* couchdb_document_put sets the ID for new documents, so need to send that back */
-		vcard_str = vcard_from_couch_document (document);
-		new_contact = e_contact_new_from_vcard (vcard_str);
-
+	if ((new_contact = put_document (couchdb_backend, document)) != NULL) {
 		e_data_book_respond_create (book, opid, GNOME_Evolution_Addressbook_Success, new_contact);
-
-		g_free (vcard_str);
 		g_object_unref (new_contact);
 	} else
 		e_data_book_respond_create (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
@@ -1083,12 +1100,25 @@ e_book_backend_couchdb_remove_contacts (EBookBackend *backend,
 	for (l = id_list; l != NULL; l = l->next) {
 		CouchDBDocument *document;
 		GError *error = NULL;
-		const gchar *uid = (const gchar *) id_list->data;
+		const gchar *uid = (const gchar *) l->data;
 
 		document = couchdb_document_get (couchdb_backend->couchdb, couchdb_backend->dbname, uid, &error);
 		if (document) {
 			if (couchdb_document_delete (document, &error))
 				deleted_ids = g_list_append (deleted_ids, (gpointer) uid);
+			else {
+				if (error != NULL) {
+					g_debug ("Error deleting document: %s", error->message);
+					g_error_free (error);
+				} else
+					g_debug ("Error deleting document");
+			}
+		} else {
+			if (error != NULL) {
+				g_debug ("Error getting document: %s", error->message);
+				g_error_free (error);
+			} else
+				g_debug ("Error getting document");
 		}
 	}
 
@@ -1107,9 +1137,8 @@ e_book_backend_couchdb_modify_contact (EBookBackend *backend,
 				       guint32 opid,
 				       const char *vcard)
 {
-	EContact *contact;
+	EContact *contact, *new_contact;
 	CouchDBDocument *document;
-	GError *error = NULL;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	contact = e_contact_new_from_vcard (vcard);
@@ -1126,8 +1155,9 @@ e_book_backend_couchdb_modify_contact (EBookBackend *backend,
 	}
 
 	/* save the contact into the DB */
-	if (couchdb_document_put (document, couchdb_backend->dbname, &error)) {
-		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_Success, contact);
+	if ((new_contact = put_document (couchdb_backend, document)) != NULL) {
+		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_Success, new_contact);
+		g_object_unref (new_contact);
 	} else
 		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
 
